package admin.rest;


import admin.base.PageInfo;
import admin.modules.chatgpt.domain.ChatgptRoleRecords;
import admin.modules.chatgpt.domain.ChatgptRoleSession;
import admin.modules.chatgpt.service.ChatgptRoleRecordsService;
import admin.modules.chatgpt.service.ChatgptRoleSessionService;
import admin.modules.chatgpt.service.dto.ChatgptRoleRecordsDto;
import admin.modules.chatgpt.service.dto.ChatgptRoleRecordsQueryCriteria;
import admin.modules.chatgpt.service.dto.ChatgptRoleSessionDto;
import admin.modules.chatgpt.service.dto.ChatgptRoleSessionQueryCriteria;
import admin.resp.ApiResponse;
import admin.util.ScreenEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Pageable;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Comparator;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author wk
 * @date 2023-11-27
 **/
@Slf4j
@RestController
@RequiredArgsConstructor
@Api(tags = "对话记录")
@RequestMapping(value = "wx/chatgptRoleRecords/")
public class ChatgptRoleRecordsController {

    private final ChatgptRoleRecordsService chatgptRoleRecordsService;
    private final ChatgptRoleSessionService chatgptRoleSessionService;

    @ApiOperation("会话查询对话记录")
    @GetMapping(value = "sessionList")
    public ApiResponse<Object> querySessionList(ChatgptRoleRecordsQueryCriteria query) {
        List<ChatgptRoleRecordsDto> chatgptRoleRecordsDtoPageInfo = chatgptRoleRecordsService.queryAll(query);
        return ApiResponse.buildSuccess(chatgptRoleRecordsDtoPageInfo);
    }


    @ApiOperation("查询会话")
    @GetMapping(value = "list")
    public ApiResponse<Object> query(ChatgptRoleSessionQueryCriteria query){
        Integer byUserName = ScreenEnum.getByUserName(query.getName());
        query.setUserId(byUserName);
        List<ChatgptRoleSessionDto> chatgptRoleSessionDtos = chatgptRoleSessionService.queryAll(query);
        return ApiResponse.buildSuccess(chatgptRoleSessionDtos!=null ? chatgptRoleSessionDtos.stream().sorted(Comparator.comparing(ChatgptRoleSessionDto::getId).reversed()).collect(Collectors.toList()) : chatgptRoleSessionDtos);
    }


    @ApiOperation("收藏会话")
    @GetMapping(value = "collect/list")
    public ApiResponse<Object> collectList(String content,String sessionId,Integer isDepth){
        List<ChatgptRoleRecords> list = chatgptRoleRecordsService.lambdaQuery()
                .eq(ChatgptRoleRecords::getMessage, content)
                .eq(ChatgptRoleRecords::getSessionId,sessionId).list();
        list.forEach(i->{
            i.setIsCollect(1);
            i.setIsDepth(isDepth);
        });
        chatgptRoleRecordsService.updateBatchById(list);
        return ApiResponse.buildSuccess();
    }

    @ApiOperation("取消收藏会话")
    @GetMapping(value = "cancel/collect/list")
    public ApiResponse<Object> cancelCollectList(Integer id){
        ChatgptRoleRecordsDto byId = chatgptRoleRecordsService.findById(id);
        if (byId == null) return ApiResponse.buildFailure("99","会话不存在");
        byId.setIsCollect(0);
        chatgptRoleRecordsService.updateById(byId);
        return ApiResponse.buildSuccess();
    }

    @ApiOperation("查询收藏会话")
    @GetMapping(value = "query/collect/list")
    public ApiResponse<Object> queryCollectList(@RequestParam String userName){
        Integer userId = ScreenEnum.getByUserName(userName);
        List<ChatgptRoleRecords> list = chatgptRoleRecordsService.lambdaQuery().eq(ChatgptRoleRecords::getIsCollect, 1).and(i -> i.eq(ChatgptRoleRecords::getSenderId, userId).or().eq(ChatgptRoleRecords::getReceiverId, userId)).list();
        return ApiResponse.buildSuccess(list.stream().sorted(Comparator.comparing(ChatgptRoleRecords::getId).reversed()).collect(Collectors.toList()));
    }

    @ApiOperation("删除会话")
    @GetMapping(value = "delete")
    public ApiResponse<Object> delete(Integer sessionId){
        chatgptRoleSessionService.removeById(sessionId);
        List<ChatgptRoleRecords> list = chatgptRoleRecordsService.lambdaQuery().eq(ChatgptRoleRecords::getSessionId, sessionId).list();
        chatgptRoleRecordsService.removeBatchByIds(list);
        return ApiResponse.buildSuccess();
    }

    @ApiOperation("大屏临时查询十条会话记录")
    @GetMapping(value = "queryTenRecords")
    public ApiResponse<Object> queryTenRecords(@RequestParam String userName) {
        Integer userId = ScreenEnum.getByUserName(userName);
        List<ChatgptRoleRecordsDto> chatgptRoleRecordsDtos = chatgptRoleRecordsService.queryTenRecords(userId);
        List<ChatgptRoleRecordsDto> collect = chatgptRoleRecordsDtos.stream().sorted(Comparator.comparing(ChatgptRoleRecordsDto::getId)).collect(Collectors.toList());
        return ApiResponse.buildSuccess(collect);
    }

    @ApiOperation("查询屏幕列表")
    @GetMapping(value = "queryScreenList")
    public ApiResponse<Object> queryScreenList() {
        return ApiResponse.buildSuccess(ScreenEnum.getScreen());
    }

}
